<?php


namespace Schaffen\WarehouseInventory\WarehouseInventory\Services;


use Schaffen\Core\Company\Repositories\BranchUserRepository;
use Schaffen\Core\Product\Repositories\ProductVarietyPackRepository;
use Schaffen\WarehouseInventory\WarehouseInventory\Managers\AccountInventoryManager;
use Schaffen\WarehouseInventory\WarehouseInventory\Managers\SnapshotInventoryManager;
use Schaffen\WarehouseInventory\WarehouseInventory\Repositories\SnapshotInventoryRepository;
use Schaffen\WarehouseInventory\WarehouseInventory\Repositories\WarehouseInventoryRepository;

/**
 * Class SnapshotService
 * @package Schaffen\WarehouseInventory\WarehouseInventory\Services
 */
class SnapshotService
{
    private SnapshotInventoryRepository $snapshot_inventory_repository;
    private SnapshotInventoryManager $snapshot_inventory_manager;
    private ProductVarietyPackRepository $product_variety_pack_repository;
    private AccountInventoryManager $account_inventory_manager;
    private BranchUserRepository $branch_user_repository;
    private WarehouseInventoryRepository $warehouse_inventory_repository;

    const IN = 1;
    const OUT = 0;

    const ENTRADA = 'ENTRADA';
    const SAIDA = 'SAIDA';
    const ORDEM_PRODUCAO = 'ORDEM_PRODUCAO';

    const SNA_VALUE_COPY = 'SNA_VALUE_COPY';
    const SNA_VALUE_KEEP = 'SNA_VALUE_KEEP';
    const SNA_VALUE_REACALCULATE = 'SNA_VALUE_REACALCULATE';

    public function __construct(SnapshotInventoryRepository $snapshot_inventory_repository,
                                SnapshotInventoryManager $snapshot_inventory_manager,
                                ProductVarietyPackRepository $product_variety_pack_repository,
                                AccountInventoryManager $account_inventory_manager,
                                BranchUserRepository $branch_user_repository,
                                WarehouseInventoryRepository $warehouse_inventory_repository)
    {

        $this->snapshot_inventory_repository = $snapshot_inventory_repository;
        $this->snapshot_inventory_manager = $snapshot_inventory_manager;
        $this->product_variety_pack_repository = $product_variety_pack_repository;
        $this->account_inventory_manager = $account_inventory_manager;
        $this->branch_user_repository = $branch_user_repository;
        $this->warehouse_inventory_repository = $warehouse_inventory_repository;

    }

    public function getInventoryProduct($warehouse_code, $pve_code)
    {
        return $this->snapshot_inventory_repository->firstOrNew([
            "sna_fk_pve_codigo" => $pve_code,
            "sna_fk_loe_codigo" => $warehouse_code,
        ], ["sna_quantidade" => 0, "sna_valor_medio" => 0]);
    }

    /**
     * @param $warehouse_code
     * @param $pve_code
     * @param $amount
     */
    public function moveTo($warehouse_code, $pve_code, $amount, $way)
    {
        $snapshot = $this->getInventoryProduct($warehouse_code, $pve_code);

        return $this->snapshot_inventory_repository->storeOrUpdate(["sna_quantidade" => $snapshot->sna_quantidade + ($way ?
                $amount : -1 * $amount)
        ],
            $snapshot->sna_codigo
        );

    }

    public function validateProductInventory($loe_codigo, $pve_codigo, $quantity)
    {
        return $this->snapshot_inventory_repository->firstOrFail([
            "sna_fk_pve_codigo" => $pve_codigo,
            "sna_fk_loe_codigo" => $loe_codigo,
            "quantity" => $quantity
        ]);
    }

    public function createAccountInventoryProductionOrder(
        int $po_code, 
        $amount, 
        int $pve_code, 
        $value, 
        int $warehouse_code,
        $way,
        $operation,
        $sna_value_mode
    )
    {
        
        $snapshot = $this->getInventoryProduct($warehouse_code, $pve_code);
        $medium_value = 0;

        switch ($sna_value_mode) {
            case self::SNA_VALUE_COPY:
                $medium_value = $value;
                break;
            case self::SNA_VALUE_KEEP:
                $medium_value = $snapshot->sna_valor_medio;
                break;
            case self::SNA_VALUE_REACALCULATE:
                $medium_value = ($snapshot->sna_quantidade * $snapshot->sna_valor_medio + $amount * $value) 
                                /($snapshot->sna_quantidade + $amount);
                break;
        }

        $values = array_filter([
            "sna_quantidade" => $snapshot->sna_quantidade + ($way ? $amount : -1 * $amount),
            "sna_valor_medio" => $medium_value
        ], fn ($value) => !is_null($value));

        $this->snapshot_inventory_repository->storeOrUpdate(
            $values,
            $snapshot->sna_codigo
        );

        return $this->account_inventory_manager->storeOrUpdate([
            "cc_fk_pve_codigo" => $pve_code,
            "cc_quantidade" => $amount,
            "cc_valor_unitario" => $value,
            "cc_fk_loe_codigo" => $warehouse_code,
            "cc_sentido" => $way,
            "cc_fk_ope_codigo" => $operation,
            "cc_codigo_operacao" => $po_code
        ]);

    }

    public function createAccountInventory(int $po_code, $amount, int $pve_code, $value, int $warehouse_code,
                                           $way, $operation)
    {
        $this->moveTo($warehouse_code, $pve_code, $amount, $way);

        return $this->account_inventory_manager->storeOrUpdate([
            "cc_fk_pve_codigo" => $pve_code,
            "cc_quantidade" => $amount,
            "cc_valor_unitario" => $value,
            "cc_fk_loe_codigo" => $warehouse_code,
            "cc_sentido" => $way,
            "cc_fk_ope_codigo" => $operation,
            "cc_codigo_operacao" => $po_code
        ]);

    }

    public function getInventoryFromBranch(int $fil_codigo)
    {
        return $this->warehouse_inventory_repository->firstOrFail([
            "loe_padrao_balcao" => 1,
            "loe_fk_fil_codigo" => $fil_codigo
        ]);
    }

    public function getProductionOrderWarehouse(int $fil_codigo)
    {

        return $this->warehouse_inventory_repository->firstOrNew([
            "loe_tipo" => "ORDEM_PRODUCAO",
            "loe_fk_fil_codigo" => $fil_codigo
        ], ["loe_descricao" => "Estoque de Ordem de Produção",
            "loe_status" => 1]);
    }

    public function getBranchUser()
    {

        return $this->branch_user_repository->firstOrFail([
            "usf_fk_usu_codigo" => auth()->user()->usu_codigo
        ]);

    }
}
